import { tabulate, updateTabulate } from './tabulate';

// https://bl.ocks.org/syntagmatic/05a5b0897a48890133beb59c815bd953
export const parallelCoordinates = (selection, props) => {
  const {
    width,
    height,
    margin,
    data,
    selectedCategory,
    selectedItem,
    mouseHoverItem,
    onTableRowMouseOver,
    columns,
    color,
    updateTable,
    filterMenuByCategory
  } = props;
	// console.log(selectedCategory)
  // let mouseHoverItem = {
  //     "Calcium (% Daily Value)": 20,
  //     "Calories": 520,
  //     "Calories from Fat": 270,
  //     "Carbohydrates": 43,
  //     "Carbohydrates (% Daily Value)": 14,
  //     "Category": "Breakfast",
  //     "Cholesterol": 250,
  //     "Cholesterol (% Daily Value)": 83,
  //     "Dietary Fiber": 3,
  //     "Dietary Fiber (% Daily Value)": 12,
  //     "Iron (% Daily Value)": 20,
  //     "Item": "Bacon, Egg & Cheese Biscuit (Large B",
  //     "Protein": 19,
  //     "Saturated Fat": 14,
  //     "Saturated Fat (% Daily Value)": 68,
  //     "Serving Size": "5.8 oz (164 g)",
  //     "Sodium": 1410,
  //     "Sodium (% Daily Value)": 59,
  //     "Sugars": 4,
  //     "Total Fat": 30,
  //     "Total Fat (% Daily Value)": 47,
  //    	"Trans Fat": null,
  //     "Vitamin A (% Daily Value)": 15,
  //    " Vitamin C (% Daily Value)": 8
  //   }
	const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  var types = {
    "Number": {
      key: "Number",
      coerce: function(d) { return +d; },
      extent: d3.extent,
      within: function(d, extent, dim) { return extent[0] <= dim.scale(d) && dim.scale(d) <= extent[1]; },
      defaultScale: d3.scaleLinear().range([innerHeight, 0])
    },
    "String": {
      key: "String",
      coerce: String,
      extent: function (data) { return data.sort(); },
      within: function(d, extent, dim) { return extent[0] <= dim.scale(d) && dim.scale(d) <= extent[1]; },
      defaultScale: d3.scalePoint().range([0, innerHeight])
    }
  };

  var dimensions = [
    {
      key: "Category",
      description: "Category",
      type: types["String"],
      axis: d3.axisLeft()
        .tickFormat(function(d,i) {
          return d;
        })
  	},
    {
      key: "Calories",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Calories from Fat",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Total Fat",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Total Fat (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Saturated Fat",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Saturated Fat (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Trans Fat",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Cholesterol",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Cholesterol (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Sodium",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Sodium (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Carbohydrates",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Carbohydrates (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Dietary Fiber",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Dietary Fiber (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Sugars",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Protein",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Vitamin A (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Vitamin C (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    },
    {
      key: "Iron (% Daily Value)",
      type: types["Number"],
      scale: d3.scaleSqrt().range([innerHeight, 0])
    }
  ];
  
  var xscale = d3.scalePoint()
    .domain(d3.range(dimensions.length))
    .range([0, innerWidth]);

  var yAxis = d3.axisLeft();

  const container = selection.selectAll('.parcoords').data([null]);
  
  const containerEnter = 
  container
  	.enter()
      .append('div')
      .attr('class', 'parcoords')
      .style("width", width)
      .style("height", height);
  
  container.exit().remove();

  // const containerEnter = container
  //   .enter().append("svg")
  //     .attr("width", width)
  //     .attr("height", height)
  //   .append("g")
  //     .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

  /*var container = d3.select(".parcoords")
      .style("width", width)
      .style("height", height);*/
  
  var svg = containerEnter.append("svg")
      .attr("width", width)
      .attr("height", height)
    .append("g")
      .attr("transform", "translate(" + margin.left + "," + margin.top + ")");
  
  var canvas = containerEnter.append("canvas")
  		.attr("class", "canvas")
      .attr("width", innerWidth)
      .attr("height", innerHeight)
      .style("margin-top", margin.top + "px")
      .style("margin-left", margin.left + "px");
  
  // console.log("canvas", canvas);
  var ctx = selection.select('canvas').node().getContext("2d");
  
  //ctx.globalCompositeOperation = 'darken';
  ctx.globalAlpha = .2;
  ctx.lineWidth = 2.5;
  
  // var output = d3.select("body").append("pre");

  var axes = svg.selectAll(".axis").data(dimensions);
  
  var axesEnter = axes
      .enter()
  			.append("g")
        .attr("class", function(d) { return "axis " + d.key.replace(/ /g, "_"); })
        .attr("transform", function(d,i) { return "translate(" + xscale(i) + ")"; });
	
  // axes.exit().remove();
  
  // shuffle the data!
  // data = d3.shuffle(data);

  data.forEach(function(d) {
    dimensions.forEach(function(p) {
      d[p.key] = !d[p.key] ? 0 : p.type.coerce(d[p.key]);
    });

    // truncate long text strings to fit in data table
    for (var key in d) {
      if (d[key] && d[key].length > 35) d[key] = d[key].slice(0,36);
    }
  });

  // type/dimension default seftting happens here
  dimensions.forEach(function(dim) {
    if (!("domain" in dim)) {
      // detect domain using dimension type's extent function
      dim.domain = d3_functor(dim.type.extent)(data.map(function(d) { return d[dim.key]; }));
    }
    if (!("scale" in dim)) {
      // use type's default scale for dimension
      dim.scale = dim.type.defaultScale.copy();
    }
    dim.scale.domain(dim.domain);
  });

  var render = renderQueue(draw).rate(50);

  ctx.clearRect(0,0,width,height);
  // ctx.globalAlpha = d3.min([0.85/Math.pow(data.length,0.3),1]);
  // console.log(ctx.globalAlpha)
  render(data);

  axes
    .merge(axesEnter)
    .selectAll('.domain').remove();
  
  axesEnter.append("g")
      .each(function(d) {
        var renderAxis = "axis" in d
          ? d.axis.scale(d.scale)  // custom axis
          : yAxis.scale(d.scale);  // default axis
        d3.select(this).call(renderAxis);
      })
    .append("text")
      .attr("class", "title")
      .attr("text-anchor", "start")
      .text(d => d.key);

  // Add and store a brush for each axis.
  axesEnter.append("g")
      .attr("class", "brush")
      .each(function(d) {
        d3.select(this).call(d.brush = d3.brushY()
          .extent([[-10,0], [10,height]])
          .on("start", brushstart)
          .on("brush", brush)
          .on("end", brush)
        )
      })
    .selectAll("rect")
      .attr("x", -8)
      .attr("width", 16);

  d3.selectAll(".axis.category .tick text")
    .style("fill", color);
  
  // tabulate(data, columns);
  // output.text(d3.tsvFormat(data.slice(0,30))).on('click', d => console.log(d));

  function project(d) {
    return dimensions.map(function(p,i) {
      // check if data element has property and contains a value
      if (
        !(p.key in d) ||
        d[p.key] === null
      ) return null;

      return [xscale(i),p.scale(d[p.key])];
    });
  };
  
  function draw(d) {
    var brushedItems = d3.selectAll(".brushed").data();
    if((!selectedCategory || d['Category'] === selectedCategory) &&
      (!selectedItem || d['Item'] === selectedItem['Item']) &&
      (!mouseHoverItem || d === mouseHoverItem)
      &&
      (brushedItems.length === 0 || brushedItems.includes(d))) {
      ctx.strokeStyle = color(d['Category']);
      ctx.beginPath();
      var coords = project(d);
    coords.forEach(function(p,i) {
      // this tricky bit avoids rendering null values as 0
      if (p === null) {
        // this bit renders horizontal lines on the previous/next
        // dimensions, so that sandwiched null values are visible
        if (i > 0) {
          var prev = coords[i-1];
          if (prev !== null) {
            ctx.moveTo(prev[0],prev[1]);
            ctx.lineTo(prev[0]+6,prev[1]);
          }
        }
        if (i < coords.length-1) {
          var next = coords[i+1];
          if (next !== null) {
            ctx.moveTo(next[0]-6,next[1]);
          }
        }
        return;
      }
      
      if (i == 0) {
        ctx.moveTo(p[0],p[1]);
        return;
      }

      ctx.lineTo(p[0],p[1]);
    });
    ctx.stroke();
      
    }
    
  }

  function brushstart() {
    // console.log('brush start');
    d3.event.sourceEvent.stopPropagation();
  }

  // Handles a brush event, toggling the display of foreground lines.
  function brush() {
    render.invalidate();
      var actives = [];
    svg.selectAll(".axis .brush")
      .filter(function(d) {
        return d3.brushSelection(this);
      })
      .each(function(d) {
        actives.push({
          dimension: d,
          extent: d3.brushSelection(this)
        });
      });

      // console.log(actives);
    var selected = data.filter(function(d) {
      if (actives.every(function(active) {
          var dim = active.dimension;
          // test if point is within extents for each active brush
          return dim.type.within(d[dim.key], active.extent, dim);
        })) {
        return true;
      }
    });

    // console.log("actives", actives);
    // show ticks for active brush dimensions
    // and filter ticks to only those within brush extents
    
    svg.selectAll(".axis")
        .filter(function(d) {
          return actives.indexOf(d) > -1 ? true : false;
        })
        .classed("active", true)
        .each(function(dimension, i) {
          var extent = extents[i];
          d3.select(this)
            .selectAll(".tick text")
            .style("display", function(d) {
              var value = dimension.type.coerce(d);
              return dimension.type.within(value, extent, dimension) ? null : "none";
            });
        });

    // reset dimensions without active brushes
    svg.selectAll(".axis")
        .filter(function(d) {
          return actives.indexOf(d) > -1 ? false : true;
        })
        .classed("active", false)
        .selectAll(".tick text")
          .style("display", null);

    ctx.clearRect(0,0,width,height);
    ctx.globalAlpha = d3.min([0.85/Math.pow(selected.length,0.3),1]);
    let brushedItems = d3.selectAll(".brushed").data();
    let selectedAndBrushedItems = selected.filter(value => -1 !== brushedItems.indexOf(value));
    
    selectedAndBrushedItems = selectedAndBrushedItems.length === 0 ? selected : selectedAndBrushedItems;
    console.log(selectedCategory)
    selectedAndBrushedItems.filter(item => item['Category'] === selectedCategory);
    // console.log(selectedAndBrushedItems)
    render(selectedAndBrushedItems);
    updateTable(selectedAndBrushedItems);
    
  };
  
  function d3_functor(v) {
    return typeof v === "function" ? v : function() { return v; };
  };
  
};