import {
  select,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom
} from 'd3';

export const scatterplot = (selection, props) => {

  const {
    xValue,
    yValue,
    xAxisLabel,
    yAxisLabel,
    circleRadius,
    margin,
    width,
    height,
    data,
    color,
    colorValue,
    selectedCategory,
    tipMouseover,
    tipMouseout,
    selectedItem,
    onItemClicked,
    highlightBrushedCircles
  } = props;

  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
	//console.log(innerWidth, innerHeight);
  const g = selection.selectAll('.container').data([null]);
  
  const gEnter = g
    .enter().append('g')
    .attr('class', 'container')
  	.attr('width', innerWidth)
  	.attr('height', innerHeight);
  
  gEnter
    .merge(g)
    .attr('transform',
      `translate(${margin.left},${margin.top})`
    );

  // console.log(margin);
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();

  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  var brush = d3.brush().extent([[0, 0], [width, height]])
  .on("brush", highlightBrushedCircles)
  .on("end", brushended),
    idleTimeout,
    idleDelay = 350;

  gEnter.append('g')
    .attr('class', 'brush')
    .call(brush);
  

  function brushended() {
    var s = d3.event.selection;
    if (!s) {
      if (!idleTimeout) return idleTimeout = setTimeout(idled, idleDelay);
      xScale.domain(d3.extent(data, xValue)).nice();
      yScale.domain(d3.extent(data, yValue)).nice();
    } else {

      xScale.domain([s[0][0], s[1][0]].map(xScale.invert, xScale));
      yScale.domain([s[1][1], s[0][1]].map(yScale.invert, yScale));
      gEnter.select(".brush").call(brush.move, null);
    }
    // zoom();
  }

  function idled() {
    idleTimeout = null;
  }

  function zoom() {

    var t = gEnter.transition().duration(750);
    selection.select(".x-axis").transition(t).call(xAxis);
    selection.select(".y-axis").transition(t).call(yAxis);
    gEnter.selectAll("circle").transition(t)
      .attr("cx", d => xScale(xValue(d)))
      .attr("cy", d => yScale(yValue(d)));
  }

  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
    .attr('class', 'y-axis');

  yAxisG.merge(yAxisGEnter)
    .call(yAxis)
    .selectAll('.domain').remove();

  yAxisGEnter.append('text')
    .attr('class', 'axis-label')
    .attr('y', -73)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .merge(yAxisG.select('.axis-label'))
    .attr('x', -innerHeight / 2)
    .text(yAxisLabel);

  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g')
    .attr('class', 'x-axis');

  xAxisG
    .merge(xAxisGEnter)
    .attr('transform', `translate(0,${innerHeight})`)
    .call(xAxis)
    .selectAll('.domain').remove();

  xAxisGEnter.append('text')
    .attr('class', 'axis-label')
    .attr('y', 75)
    .attr('fill', 'black')
    .merge(xAxisG.select('.axis-label'))
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);

  let circles = g.merge(gEnter)
    .selectAll('circle').data(data);
  let circlesEnter = circles.enter().append('circle')
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', 0)
      .on('mouseover', tipMouseover)
      .on('mouseout', tipMouseout)
      .on('click', onItemClicked)
    .merge(circles)
      .transition().duration(2000)
      .delay((d, i) => i * 10)
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', d => (!selectedCategory || selectedCategory === colorValue(d))
          ? circleRadius : 0)
      .attr('fill', d => color(colorValue(d)))
      .attr('opacity', d =>
        (selectedCategory === colorValue(d))
          ? 1
          : 0.1
      )
  		.attr("stroke-width", d => selectedItem === d
          ? 4
          : 0)
  		.attr("stroke", "black");

  /*selection.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(title);*/

};